.8086
	include	casm.inc

	MAX	equ	07fffh
	MIN	equ	0


	update_region_table	MACRO
	local	_1,_2,_3,_4
		cmp	di,ymax
		jbe	_1
			mov	ymax,di
		_1:
		cmp	di,ymin
		jae	_2
			mov	ymin,di
		_2:
		mov	bx,di
		shl	bx,1
		mov	ax,lx

		cmp	ax,xmin[ bx ]
		jae	_3
			mov	xmin[bx],ax
		_3:
		cmp	ax,xmax[ bx ]
		jbe	_4
			mov	xmax[ bx ],ax
		_4:
	ENDM


;****************************************************************************
CODE	segment	byte public 
	assume cs: CODE


	xmin		dw	480	dup	(?)
	xmax		dw	480	dup 	(?)
	ymin		dw			(?)
	ymax		dw			(?)
	curry		dw			(?)
	lx		dw			(?)
	sum		dw			(?)
	count		dw			(?)
	count_p		dw			(?)
	;...............................................................
	; This procedure is used to fill polygon was created in any way.

	start_c_proc	fill_table_region
		mov	ax,ymin
		mov	curry,ax

		poly_scan_loop:
			mov	bx,curry
			shl	bx,1

			push    xmin[ bx ]
                        push	curry
			push    xmax[ bx ]
			push	curry
			call	far	ptr rect_fill
			
			inc	curry
			mov	ax,curry
		cmp	ax,ymax
		jbe	poly_scan_loop
	end_c_proc	fill_table_region

	;..............................................
	;	This procedure is used when new region is
	;	up to be created
	start_c_proc	refresh_table
		mov	ymin,MAX
		mov	ymax,MIN
		mov	cx,480
		mov	bx,0
		refresh_loop:
			mov	xmin[ bx ],MAX
			mov	xmax[ bx ],MIN
			add	bx,2
		loop	refresh_loop
	end_c_proc	refresh_table



	;........................................................
	;	This procedure helps to build areas adding linear
	;	border to existing area.

	start_c_proc	set_table_line
	cli

	mov	bx,[bp+12]                       ;loads coordinates to registers
	mov	ax,[bp+10]                       ;
	mov	cx,[bp+8]                        ;
	mov	dx,[bp+6]                        ;

	cmp 	ax,dx                           ;if   y > y1
	jbe	@1
		xchg	ax,dx
		xchg 	bx,cx
	@1:

	mov	di,ax                           ;di=y
	mov	lx,bx				;lx=x

	sub	dx,ax                           ;si=dely
	mov	si,dx

	mov	dx,cx                           ;calculate delx=|x1-x|
	sub	dx,bx
	jae	@2
		neg	dx			;dx=|x1-x|
	@2:
	mov     bp,dx                           ;bp=delx


	cmp	bp,si                           ;calculate line-drawing mode
	jb	@4	                        ;if dx>=dy horizontal drawing
		mov	sum,bp                  ;sum=delx
		cmp	cx,bx                   ;x1 >= x ?
		$IF     GE                      ;
			        mov	dx,offset  h_x1_ge_x
		$ELSE
			        mov	dx,offset  h_x1_ls_x
		$ENDIF
		jmp	@5
	@4:
		mov	sum,si                  ;sum=dely
		cmp     cx,bx                   ;x1 >= x ?
		$IF     GE
			        mov	dx,offset  v_x1_ge_x
		$ELSE
			        mov	dx,offset  v_x1_ls_x
		$ENDIF
	@5:


	mov     cx,sum				;at first  count=sum+1
	inc     cx
	mov	count,cx

	jmp     dx				;jump to current line type
	;------------horizontal case-------------
	h_x1_ge_x:
		update_region_table

		sub  sum,si                     ;sum-=dy
		$IF  LE                         ;if  sum <= 0
			inc  di			;y++
			add  sum,bp             ;sum+=dx
		$ENDIF
		inc	lx			;x++
	dec	count
	jnz	h_x1_ge_x                       ;one more point?
	jmp     set_line_exit
	;---------------------------------------

	h_x1_ls_x:
        	update_region_table
		sub     sum,si                  ;sum-=dy
		$IF	LE			;if  sum <= 0
			inc  di			;y++
			add  sum,bp             ;sum+=dx
		$ENDIF
		dec	lx
	dec	count
	jnz	h_x1_ls_x                       ;one more point ?

	jmp     set_line_exit
	;-------vertical  case-------------------
	v_x1_ge_x:
                update_region_table

		sub	sum,bp                   ;sum-=dx
		$IF	LE                      ;if  sum <= 0
			add   sum,si             ;sum+=dy
			inc   lx		;x++
		$ENDIF
		inc     di			;y++

	dec	count
	jnz	v_x1_ge_x
	jmp	set_line_exit

	;----------------------------------------
	v_x1_ls_x:
		update_region_table

		sub     sum,bp			;sum-=dx
		$IF	LE			;if  sum <= 0
			add	sum,si		;sum+=dy
			dec	lx		;x--
		$ENDIF
		inc     di			;y++
	dec	count
	jnz	v_x1_ls_x

	set_line_exit:
	sti
	end_c_proc    set_table_line,8

	;...................................................
	;	This procedure is used to fill polygon region
	;	defined by a set of boundaries.
	;	First parameter is a pointer to  a set consequtive
	;	points wich define polygon boundary.The second one
	;	is a number of points to be used.
	start_c_proc	fill_poly
		call	far  ptr refresh_table
		lds	si,dword ptr [bp+8]		;si points to first pnt
		mov	di,si				;di points to curr pnt

		mov	ax,[bp+6]
		mov	count_p,ax
		dec	count_p

		set_poly_loop:
			push	[di]
			push	[di+2]
			push	[di+4]
			push	[di+6]
			call	far ptr set_table_line
			
			add	di,4			;advance to next point
		dec	count_p				;one more point
		jnz	set_poly_loop

		push	[di]
		push	[di+2]
		push	[si]
		push	[si+2]
		call	far ptr set_table_line		;add last boundary
		call	far ptr	fill_table_region

	end_c_proc	fill_poly,6


	;..........................................................
	;	This procedure is used to make region table visible
	;	from another modules.

	start_c_proc	get_poly_table_addr
		mov	ax,offset xmin
		mov	dx,cs
	end_c_proc	get_poly_table_addr


CODE	ends
;*****************************************************************************
	extrn	rect_fill

	public	set_table_line
	public	fill_table_region
	public	get_poly_table_addr
	public	refresh_table
	public	fill_poly

END











